#include <stdafx.h>
#include "../MultiKeyboardDll/MultiKeyboardDll.h"
#include <windows.h>

HANDLE mutex;

bool CheckIfAllowedToRun();
bool LoadKeyboardLayoutPairs();
bool ReadCharacter(HANDLE h, PWCHAR chr);

int APIENTRY _tWinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)
{
	MSG msg;

	if(!CheckIfAllowedToRun())
		return 1;

	if(!LoadKeyboardLayoutPairs())
		return 1;

	if(!InstallKeyboardHook())
		return 1;

	while (GetMessage(&msg, NULL, 0, 0)) 
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return (int) msg.wParam;
}

bool CheckIfAllowedToRun()
{
	OSVERSIONINFO ver;

	ver.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	GetVersionEx(&ver);

	if(!(
			ver.dwPlatformId == VER_PLATFORM_WIN32_NT && 
			(
				(ver.dwMajorVersion == 5 && ver.dwMinorVersion >= 1) || ver.dwMajorVersion >= 6
			)
		)
	  )
	{
		MessageBox(0, L"Only running on XP and higher", 0, MB_OK);
		return false;
	}
	mutex = ::CreateMutex(NULL, TRUE, L"MultiKeyboardNamedMutex");

	int ret = ::WaitForSingleObject(mutex, 100);

	if(mutex == NULL || !(ret == WAIT_OBJECT_0 || ret == WAIT_ABANDONED))
	{
		MessageBox(0, L"Program is already running", 0, MB_OK);
		return false;
	}

	return true;
}

bool ReadLine(HANDLE h, PWCHAR line, int maxLen);

bool LoadKeyboardLayoutPairs()
{
	PWCHAR pPath = (PWCHAR)malloc(1024*sizeof(pPath[0]));

	if(GetModuleFileName(NULL, pPath, 1022) == ERROR_INSUFFICIENT_BUFFER)
	{
		return false;
	}

	wcscpy_s(wcsrchr(pPath, '\\') + 1, 1024, L"MultiKeyboard.conf");

	HANDLE hConfigFile = CreateFile(pPath, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
	if(hConfigFile!= INVALID_HANDLE_VALUE)
	{
		WCHAR line[256];
		WCHAR keyboardName[256];
		DWORD layoutCode;

		DWORD lineCnt = 0;
		bool result = false;
		do
		{
			result = ReadLine(hConfigFile, line, 256);
			if(result)
			{
				PWCHAR eqMark = wcschr(line, L'=');
				if(eqMark!= NULL)
				{
					*eqMark = L' ';
					if(swscanf_s(line, L"%s %d", keyboardName, 256, &layoutCode) == 2)
					{
						wcscpy_s(KNLCPairs[lineCnt].KeyboardName, 256, keyboardName);
						KNLCPairs[lineCnt].LayoutCode = layoutCode;
						lineCnt++;
					}
					else
					{
						result = false;
					}
				}
				else
				{
					result = false;
				}
			}
		} while(result && lineCnt < MAXPAIRS);
		numKNLCPair = lineCnt;

		CloseHandle(hConfigFile);
	}

	return true;
}

bool ReadLine(HANDLE h, PWCHAR line, int maxLen)
{
	BYTE b;
	DWORD r;
	int pos = 0;

	line[0] = 0;

	do
	{
		if(!ReadFile(h, &b, 1, &r, NULL))
		{
			return false;
		}
		if(r == 0)
		{
			return false;
		}
	} while(b == 13 || b == 10);

	line[pos] = b;
	line[pos+1] = 0;
	pos++;
	do
	{
		if(!ReadFile(h, &b, 1, &r, NULL))
		{
			return false;
		}
		if(r == 0)
		{
			return true;
		}
		line[pos] = b;
		line[pos+1] = 0;
		pos++;
		if(pos >= maxLen - 1)
		{
			return false;
		}
	} while(b!= 13 && b!= 10);

	return true;
}
