// MultiKeyboardDll.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include "MultiKeyboardDll.h"
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>

#pragma data_seg(".HOOKDATA")//Shared data among all instances

HHOOK hook = NULL;
MULTIKEYBOARDDLL_API DWORD numKNLCPair = 0;
MULTIKEYBOARDDLL_API KeyboardNameLayoutCodePair KNLCPairs[MAXPAIRS] = {{L"", 0}};

#pragma data_seg()
#pragma comment(linker, "/SECTION:.HOOKDATA,RWS")//linker directive

struct KeyboardHandleLayoutCodePair
{
	HANDLE KeyboardHandle;
	DWORD LayoutCode;
};

DWORD numKHLCPair = 0;
HANDLE prevKeyboardHandle = NULL;
KeyboardHandleLayoutCodePair KHLCPairs[MAXPAIRS] = {{0,0}};

HINSTANCE hInstance = NULL;

#ifdef _MANAGED
#pragma managed(push, off)
#endif

void GeneratePairs();

BOOL APIENTRY DllMain( HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
					 )
{
	if(ul_reason_for_call == DLL_PROCESS_ATTACH)
	{
		hInstance = (HINSTANCE)hModule;

		numKHLCPair = 0;
		prevKeyboardHandle = NULL;

		GeneratePairs();

		RAWINPUTDEVICE rid;
		rid.usUsagePage = 1;
		rid.usUsage = 6;
		rid.dwFlags = 0;
		rid.hwndTarget = 0;
		RegisterRawInputDevices(&rid, 1, sizeof(rid));
	}

    return TRUE;
}

#ifdef _MANAGED
#pragma managed(pop)
#endif

LRESULT CALLBACK hookproc(int ncode, WPARAM wParam, LPARAM lParam)
{
	if(ncode == HC_ACTION)
	{
		MSG *msg = (MSG *)lParam; 
		if(msg->message == WM_INPUT)
		{
			HRAWINPUT hRawInput = (HRAWINPUT)msg->lParam;
			UINT needed = 0;
			if(GetRawInputData(hRawInput, RID_HEADER, NULL, &needed, sizeof(RAWINPUTHEADER)) == 0)
			{
				LPRAWINPUT lpRawInput = (LPRAWINPUT)malloc(needed);
				if(GetRawInputData(hRawInput, RID_HEADER, lpRawInput, &needed, sizeof(RAWINPUTHEADER)) == needed)
				{
					if(lpRawInput->header.dwType == RIM_TYPEKEYBOARD)
					{
						HANDLE keyboardHandle = lpRawInput->header.hDevice;
						if(prevKeyboardHandle!= keyboardHandle)
						{
							for(DWORD h = 0;h < numKHLCPair;h++)
							{
								if(keyboardHandle == KHLCPairs[h].KeyboardHandle)
								{
									WCHAR loc[9];

									wsprintf(loc, L"%08x", KHLCPairs[h].LayoutCode);
									HKL layout = LoadKeyboardLayout(loc, KLF_REPLACELANG);
								
									ActivateKeyboardLayout(layout, 0);
									break;
								}
							}

							prevKeyboardHandle = keyboardHandle;
						}
					}
				}
				free(lpRawInput);
			}
		}
	}

	return CallNextHookEx(hook, ncode, wParam, lParam);
}

void GeneratePairs()
{
	PRAWINPUTDEVICELIST pDevList;
	UINT numInputDevices;

	if(GetRawInputDeviceList(NULL, &numInputDevices, sizeof(pDevList[0])) == 0)
	{
		pDevList = (PRAWINPUTDEVICELIST)malloc(sizeof(pDevList[0]) * numInputDevices);
		GetRawInputDeviceList(pDevList, &numInputDevices, sizeof(pDevList[0]));

		for(DWORD i = 0;i < numInputDevices && i < MAXPAIRS;i++)
		{
			if(pDevList[i].dwType == RIM_TYPEKEYBOARD)
			{
				UINT numDevName;
				if(GetRawInputDeviceInfo(pDevList[i].hDevice, RIDI_DEVICENAME, 0, &numDevName) == 0)
				{
					PWCHAR pDevName;
					pDevName = (PWCHAR)malloc(numDevName * sizeof(pDevName[0]));
					if(GetRawInputDeviceInfo(pDevList[i].hDevice, RIDI_DEVICENAME, pDevName, &numDevName))
					{
						PWCHAR ctx = NULL;
						PWCHAR p1 = wcstok_s(pDevName+4, L"#", &ctx);
						PWCHAR p2 = wcstok_s(0, L"#", &ctx);
						PWCHAR p3 = wcstok_s(0, L"#", &ctx);
						WCHAR kbName[1024];
						wsprintf(kbName, L"%s#%s#%s", p1, p2, p3);

						bool foundHandle = false;
						DWORD foundPos = -1;
						for(DWORD h = 0;h < numKNLCPair;h++)
						{
							if(wcscmp(kbName, KNLCPairs[h].KeyboardName) == 0)
							{
								foundPos = h;
								foundHandle = true;
								break;
							}
						}

						if(foundHandle)
						{
							KHLCPairs[numKHLCPair].KeyboardHandle = pDevList[i].hDevice;
							KHLCPairs[numKHLCPair].LayoutCode = KNLCPairs[foundPos].LayoutCode;
							numKHLCPair++;
						}
					}
					free(pDevName);
				}
			}
		}
		free(pDevList);
	}
}


MULTIKEYBOARDDLL_API bool InstallKeyboardHook()
{
	hook = SetWindowsHookEx(WH_GETMESSAGE, hookproc, hInstance, NULL);

	if(hook == NULL)
	{
		MessageBox(0, L"Unable to install Keyboard Hook", L"Error", MB_OK);
		return false;
	}

	return true;
}

MULTIKEYBOARDDLL_API bool UnInstallKeyboardHook(void)
{
	UnhookWindowsHookEx(hook);

	return true;
}
