// MultiKeyboardList.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>

void PrintKeyboardDevices();
void PrintLayouts();

int _tmain(int argc, _TCHAR* argv[])
{
	wprintf(L"Keyboard device codes and names:\n");
	PrintKeyboardDevices();
	wprintf(L"\n");

	wprintf(L"Keyboard layouts:\n");
	PrintLayouts();
	wprintf(L"\n");
	wprintf(L"Created by Krisztian Pocza (kpocza@kpocza.net)\n");
	wprintf(L"Licence: BSD-like\n");

	return 0;
}

void PrintKeyboardDevices()
{
	PRAWINPUTDEVICELIST pDevList;
	UINT numInputDevices;
	if(GetRawInputDeviceList(NULL, &numInputDevices, sizeof(pDevList[0])) == 0)
	{
		pDevList = (PRAWINPUTDEVICELIST)malloc(sizeof(pDevList[0]) * numInputDevices);
		GetRawInputDeviceList(pDevList, &numInputDevices, sizeof(pDevList[0]));

		for(unsigned int i = 0;i < numInputDevices;i++)
		{
			if(pDevList[i].dwType == RIM_TYPEKEYBOARD)
			{
				UINT numDevName;
				if(GetRawInputDeviceInfo(pDevList[i].hDevice, RIDI_DEVICENAME, 0, &numDevName) == 0)
				{
					PWCHAR pDevName;
					pDevName = (PWCHAR)malloc(numDevName * sizeof(pDevName[0]));
					if(GetRawInputDeviceInfo(pDevList[i].hDevice, RIDI_DEVICENAME, pDevName, &numDevName))
					{
						PWCHAR ctx = NULL;
						PWCHAR p1 = wcstok_s(pDevName+4, L"#", &ctx);
						PWCHAR p2 = wcstok_s(0, L"#", &ctx);
						PWCHAR p3 = wcstok_s(0, L"#", &ctx);
						WCHAR regPath[1024];
						wsprintf(regPath, L"SYSTEM\\CurrentControlSet\\Enum\\%s\\%s\\%s", p1, p2, p3);

						HKEY regHandle;
						if(RegOpenKeyW(HKEY_LOCAL_MACHINE, regPath, &regHandle) == ERROR_SUCCESS)
						{
							DWORD nRegClassLen;
							if(RegQueryValueEx(regHandle, L"Class", 0, 0, NULL, &nRegClassLen) == ERROR_SUCCESS)
							{
								PBYTE pRegClass = (PBYTE)malloc(nRegClassLen * sizeof(pRegClass[0]));
								if(RegQueryValueEx(regHandle, L"Class", 0, 0, pRegClass, &nRegClassLen) == ERROR_SUCCESS)
								{
									if(_wcsnicmp((PWCHAR)pRegClass, L"Keyboard", nRegClassLen*2) == 0)
									{
										DWORD nRegDeviceDescLen;
										if(RegQueryValueEx(regHandle, L"DeviceDesc", 0, 0, NULL, &nRegDeviceDescLen) == ERROR_SUCCESS)
										{
											PBYTE pRegDeviceDesc = (PBYTE)malloc(nRegDeviceDescLen * sizeof(pRegDeviceDesc[0]));
											if(RegQueryValueEx(regHandle, L"DeviceDesc", 0, 0, pRegDeviceDesc, &nRegDeviceDescLen) == ERROR_SUCCESS)
											{
												wprintf(L"%s#%s#%s: %s\n", p1, p2, p3, (PWCHAR)pRegDeviceDesc);
											}
											free(pRegDeviceDesc);
										}
									}
								}
								free(pRegClass);
							}
							RegCloseKey(regHandle);
						}
					}
					free(pDevName);
				}
			}
		}
		free(pDevList);
	}
}

void PrintLayouts()
{
	int numEntries = GetKeyboardLayoutList(0, NULL);

	if(numEntries > 0)
	{
		HKL *pEntries = (HKL *)malloc(sizeof(pEntries[0]) * numEntries);
		if(GetKeyboardLayoutList(numEntries, pEntries) == numEntries)
		{
			for(int i = 0;i < numEntries;i++)
			{
				WORD langId = (WORD)(((DWORD)pEntries[i])>>16);
				WCHAR regPath[1024];
				wsprintf(regPath, L"SYSTEM\\CurrentControlSet\\Control\\Keyboard Layouts\\%08x", langId);
				
				HKEY regHandle;
				if(RegOpenKeyW(HKEY_LOCAL_MACHINE, regPath, &regHandle) == ERROR_SUCCESS)
				{
					DWORD nRegLayoutLen;
					if(RegQueryValueEx(regHandle, L"Layout Text", 0, 0, NULL, &nRegLayoutLen) == ERROR_SUCCESS)
					{
						PBYTE pRegLayout = (PBYTE)malloc(nRegLayoutLen * sizeof(pRegLayout[0]));
						if(RegQueryValueEx(regHandle, L"Layout Text", 0, 0, pRegLayout, &nRegLayoutLen) == ERROR_SUCCESS)
						{
							wprintf(L"%d: %s\n", langId, (PWCHAR)pRegLayout);
						}
						free(pRegLayout);
					}
					RegCloseKey(regHandle);
				}
			}
		}
		free(pEntries);
	}
}